@@ -16,11 +16,11 @@ newtype Zero = Zero () deriving (Eq, Show, NFData, Typeable, Hashable, Binary)
 
 type instance RuleResult Zero = Zero
 
-main = shakeTest_ test $ do
+main = testBuild test $ do
     addOracle $ \Zero{} -> do
         liftIO $ createDirectoryRecursive "dir"
         liftIO $ setCurrentDirectory "dir"
-        return $ Zero ()
+        pure $ Zero ()
 
     "changedir" %> \out -> do
         Zero () <- askOracle $ Zero ()
@@ -49,6 +49,13 @@ main = shakeTest_ test $ do
         writeFile' "createonce" "Y"
         writeFile' out ""
 
+    "recordtwice" %> \out -> do
+        alwaysRerun
+        trackWrite ["recordtwice_"]
+        trackWrite ["recordtwice_"]
+        writeFile' "recordtwice_" ""
+        writeFile' out ""
+
     "listing" %> \out -> do
         writeFile' (out <.> "ls1") ""
         getDirectoryFiles "" ["//*.ls*"]
@@ -102,6 +109,16 @@ main = shakeTest_ test $ do
         need ["tracker-source.c"]
         cmd AutoDeps "gcc" ["-c", "tracker-source.c", "-o", out]
 
+    "../lint2/tracker-relative" %> \out ->
+        writeFile' out "tracker-relative"
+    "tracker-relative1" %> \out -> do
+        need ["../lint2/tracker-relative"]
+        access "../lint2/tracker-relative"
+        writeFile' out "tracker-relative"
+    "tracker-relative2" %> \out -> do
+        access "../lint2/tracker-relative"
+        writeFile' out "tracker-relative"
+
     where gen t f = cmd Shell "echo" t ">" (toNative f) :: Action ()
           access f = if isWindows
                      then cmd_ Shell "type" (toNative f) "> nul"
@@ -120,6 +137,7 @@ test build = do
     crash ["--clean","cdir.1","pause.2","-j2"] ["output","lint","current directory has changed"]
     crash ["existance"] ["changed since being depended upon"]
     crash ["createtwice"] ["changed since being depended upon"]
+    build ["recordtwice"]
     crash ["listing"] ["changed since being depended upon","listing.ls2"]
     crash ["--clean","listing","existance"] ["changed since being depended upon"]
     crash ["needed1"] ["'needed' file required rebuilding"]
@@ -136,3 +154,5 @@ test build = do
         crash ["tracker-read3"] ["depended upon after being used","tracker-source2"]
         build ["tracker-compile.o"]
         build ["tracker-compile-auto.o"]
+        build ["tracker-relative1"]
+        crash ["tracker-relative2"] ["lint2/tracker-relative"]
